#!/usr/bin/perl 
#
# $Id: maurice-irc.pl,v 1.20 2009/07/29 09:53:50 cbellot Exp $
#
# maurice-irc
# Copyright (C) 2006 Cyril Bellot
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#


no utf8;
use Net::IRC; 

#######################################################################
# Default vars

our $server;
our $port = 6667;
our $mynick;
our $pass;
our $ircname;
our $username;
our $channel;
our $boss;
our @blacklist;
our $typespeed = 100;

my $configfile = $ENV{'HOME'}."/.maurice-irc.conf";

if ($#ARGV == 0) {
    $configfile = $ARGV[0];
    dprint("## main : using $configfile");
}

die "No $configfile file " if ! -f $configfile;
do $configfile;

our $here = $ENV{"PWD"};
our %commands;
our $mainhelp;

my $version = 'maurice-irc CVS $Revision: 1.20 $';

#######################################################################
# Main

my $irc = new Net::IRC; 
my $conn;
&connect();
&loadallmodules();
$irc->start(); 

#######################################################################
# Main functions

sub dprint {
    my $say = shift;
    print "$say\n";
}

sub loadallcommands {
    our %commandslist;
    dprint("## main::loadallcommands : Loading all commands"); 
    do "commands.conf";
    %commands = %commandslist;
}

sub loadallmodules {
    dprint("## main::loadallmodules : Loading all modules"); 
    &loadallcommands();
    my %allready;
    while (my ($cmd, $data) = each %commands) {
	my $regx = $data->{regexp};
	my $module = $data->{module};
	if (defined $allready{$module}) {
	    $data->{ln} = $allready{$module};
	} else {
	    require "./modules/$module.pm";
	    my $modulename = "modules::".$module."::new";
	    $allready{$module} = $data->{ln} = &$modulename();
	    $data->{ln}->load();
	}
    }
}

sub unloadallmodules {
    dprint("## main::unloadallmodules: Unloading all modules"); 
    my @allready = ();
    while (my ($cmd, $data) = each %commands) {
	my $module = $data->{module};
	if (! grep(/$module/, @allready)) {
	    push @allready, $module;
	    $data->{ln}->unload();
	    delete $INC{'./modules/'.$module.'.pm'};
	}
    }
    undef %commands;
    delete $INC{'commands.conf'};
}

sub reloadallmodules {
    dprint("## main::reloadallmodules");
    &unloadallmodules();
    &loadallmodules();
}


sub connect {
    dprint("== main::connect : connecting..."); 
    $conn = $irc->newconn( 
	'Server'      => $server, 
	'Port'        => $port, 
	'Nick'        => $mynick, 
	'Ircname'     => $ircname, 
	'Username'    => $username 
    ); 
    $conn->add_handler('endofmotd', \&on_connect);
    $conn->add_handler('public', \&on_public);
    $conn->add_handler('join', \&on_join);
    $conn->add_handler('part', \&on_part);
    $conn->add_handler('notice', \&on_notice);
    $conn->add_handler('msg', \&on_private);
    $conn->add_handler('caction', \&on_caction);
    $conn->add_handler('cversion', \&on_cversion);
    $conn->add_handler('cping', \&on_cping);
    $conn->add_handler('ctime', \&on_ctime);
    $conn->add_handler('disconnect', \&on_disconnect);
}

sub notice {
    my ($to, $what) = @_; 
    for my $l (split /\n/, $what) {
	$l =~ s/^$/ /;
	dprint "** main::notice : noticing to $to : $l";
	$conn->notice($to, $l); 
    }
}
sub directsay {
    my ($to, $what) = @_; 
    for my $l (split /\n/, $what) {
	$l =~ s/^$/ /;
	dprint "** main::directsay : saying to $to : $l";
	$conn->privmsg($to, $l); 
    }
}
sub say {
    my ($to, $what) = @_; 
    dprint "** main::say : delaying ".(length($what)/$typespeed)."s";
    # Delayed say (like if we were typing...)
    sleep(length($what)/$typespeed);
    &directsay($to,$what);
}

sub process_commands {
    my ($conn, $event, $text) = @_; 
    while (my ($cmd, $data) = each %commands) {
	my $regx = $data->{regexp};
	my $ln = $data->{ln};
	my $module = $data->{module};
	my $cmd = $data->{cmd};
	if ($text =~ /$regx/) {
	    dprint("** main::process_commands : Launching ".$module."::".$cmd);
	    $ln->$cmd($conn, $event, $text, $channel);
	}
    }
}

sub process_catchall_commands {
    my ($conn, $event, $text) = @_; 
    while (my ($cmd, $data) = each %commands) {
	my $regx = $data->{regexp};
	my $ln = $data->{ln};
	my $module = $data->{module};
	my $cmd = $data->{cmd};
	if ($regx eq ".*") {
	    dprint("** main::process_catchall_commands : ($regx) Launching ".
		$module."::".$cmd);
	    $ln->$cmd($conn, $event, $text, $channel);
	}
    }
}


#######################################################################
# events

sub on_connect { 
    my ($conn, $event) = @_; 
    $conn->join($channel); 
    dprint("== main::on_connect : Identify : $mynick #######"); 
    #say($channel, "I'm back ($version)"); 
    $conn->{'connected'} = 1; 
    $conn->privmsg('nickserv', 'IDENTIFY '.$pass); 
}

sub on_disconnect { 
    my ($conn, $event) = @_; 
    dprint("== main::on_disconnect : trying to reconnect"); 
    $irc->removeconn($conn);
    &connect();
}

sub on_public { 
    my ($conn, $event) = @_; 
    my $text = $event->{'args'}[0]; 
    dprint("== main::on_public : <".$event->{'nick'}."> $text"); 
    &process_commands($conn, $event, $text);
}

sub on_private { 
    my ($conn, $event) = @_; 
    my $text = $event->{'args'}[0]; 
    dprint("== main::on_private : <".$event->nick()."> $text"); 
    if (!grep(/$event->nick()/i,@blacklist)) {
	&process_commands($conn, $event, $text);
    }
}

sub on_join { 
    my ($conn, $event) = @_; 
    dprint("== main::on_join : ".$event->{'nick'}." has joined"); 
    if ($event->{'nick'} ne $mynick) {
	&process_catchall_commands($conn, $event, "-!- has joined -!-");
    }
}

sub on_part { 
    my ($conn, $event) = @_; 
    dprint("== main::on_part : ".$event->{'nick'}." has quit"); 
    &process_catchall_commands($conn, $event, "-!- has quit -!-");
}

sub on_notice { 
    my ($conn, $event) = @_; 
    my $text = $event->{'args'}[0]; 
    dprint("== main::on_notice : <".$event->nick()."> $text"); 
    if ($event->nick() =~ /$boss/) {
	if ($text =~ /^!reload$/) {
	    &reloadallmodules(); 
	    &notice($boss,"all modules reloaded, boss!");
	}
	if ($text =~ /^!quit$/) {
	    directsay($channel, "Bye bye!"); 
	    &unloadallmodules();
	    exit;
	}
    }
}

sub on_caction { 
    my ($conn, $event) = @_; 
    my $nick = $event->nick; 
    my $text = $event->{'args'}[0]; 
    dprint("* $nick $text"); 
}

sub on_cversion { 
    my ($conn, $event) = @_; 
    my $nick = $event->nick; 
    dprint("== main::on_cversion : request from $nick"); 
    $conn->ctcp_reply($nick, 'VERSION '.$version); 
    $conn->privmsg($channel, "$nick: WTF with my version ?");
}

sub on_cping { 
    my ($conn, $event) = @_; 
    my $nick = $event->nick; 
    my $timestamp = $event->{'args'}[0]; 
    dprint("== main::on_cping : request from $nick"); 
    $conn->ctcp_reply($nick, 'PING '.$timestamp); 
}

sub on_ctime { 
    my ($conn, $event) = @_; 
    my $date = localtime; 
    my $nick = $event->nick; 
    dprint("== main::on_ctime : request from $nick"); 
    $conn->ctcp_reply($nick, 'TIME '.$date); 
}

